/* 
 * File:   recollparser.cpp
 * Author: david
 * 
 * Created on November 10, 2011, 11:00 AM
 */

#include <QByteArray>
#include <QString>
#include <QDebug>

#include "recollparser.h"

recollparser::recollparser( QStringList data ) {
    copyData( data );
}

void recollparser::copyData( QStringList& data ) {
    // copy this data to instance;
    m_data = data;
    m_nParsed = data.count();
    // DO THIS FOR PROTECTION!!
    while ( m_nParsed++ < NTYPES )
        m_data << "";
    init( 0 );
}

recollparser::recollparser( QByteArray input, int UseMax ) { 
    int n;
    QByteArray fromBase64;
    QList< QByteArray > raw;
    
    //qDebug() << "recoll parser constructor input:" << input;
    raw = input.split( ' ' );
    m_nParsed = raw.count();
    //qDebug() << "recoll parser constructor data count" << m_nParsed;
    
    for ( n = 0; n < m_nParsed; ++n ) {
        fromBase64 = QByteArray::fromBase64( raw[ n ] );
        m_data << QString::fromLocal8Bit( fromBase64 );
        //qDebug() << "recoll parse data item: " << n << "-" << m_data[ n ];
    }
    // DO THIS FOR PROTECTION!!
    while ( n++ < NTYPES )
        m_data << ""; 
    
    init( UseMax );
}

void recollparser::init( int UseMax ) {
    m_URL =  QUrl( m_data[ eURL ] );
    m_file.setFile( m_URL.toLocalFile() );
    
    if  ( UseMax > 0 )
        m_nMax = UseMax + 1;
    else
        m_nMax = RECOLLMAX + 1;        
}

QString recollparser::fieldList() {
    QString ret;
    for ( int n = 0; n < NTYPES; ++n ) {
        if  ( n > 0 )
            ret += ' ';
        ret += CMDS[ n ];
    }
    
    return ret;
}

int recollparser::recollRelevInt() {
    QString str = m_data[ eRELEVANCE ];
    str.remove( QRegExp( "\\D+") );
    return str.toInt();

}

double recollparser::recollRelevfloat() {
    int numb = recollRelevInt();

    return (double) numb / 100.0;
}

double recollparser::mapRelevToRange( double min, double max ) {
    return mapFloatToRange( recollRelevfloat() , min, max );
}

double recollparser::mapFloatToRange(double rank, double min, double max) {
    return min +  rank * ( max - min - 0.1 );
}

double recollparser::mapSeqToRange(int seq, double min, double max) {
    return mapFloatToRange( 1.0 - (double) ( seq + 1 ) / (double) m_nMax, min, max );
}

bool recollparser::ipathIsNumber() {
    return ipathToNumber() >= 0;
}

bool recollparser::ipathIsNumberColonNumber() {
    if  ( !hasIPath() )
        false;

    bool ok;
    QString ip = m_data[ eIPATH ];
    ip.remove(':').toInt( &ok, 10 );

    return ok;
}

int recollparser::ipathToNumber() {
    if  ( !hasIPath() )
        return -1;

    bool ok;
    int n = m_data[ eIPATH ].toInt( &ok, 10 );

    return ok ? n : -1;
}

QDate recollparser::date() {
    bool ok;
    //qDebug() << parserData[ eDATE ];
    ulong uldate = m_data[ eDATE ].toULong( &ok, 10 );
    //qDebug() << uldate;
    QDateTime filedate = QDateTime::fromTime_t( uldate );
    return ok ? filedate.date() : QDate();
}
