/* 
 * File:   recollparser.h
 * Author: david
 *
 * Created on November 10, 2011, 11:00 AM
 */

#ifndef RECOLLPARSER_H
#define	RECOLLPARSER_H

#include <QFileInfo>
#include <QStringList>
#include <QUrl>
#include <QRegExp>
#include <QDate>

typedef enum { eMIMETYPE, eURL, eFILENAME, eTITLE, eRELEVANCE, eABSTRACT,
                        eAUTHOR, eDATE, eIPATH } eRECOLLPARSER;
static const QString CMDS[] = { "mtype", "url", "filename", "title", "relevancyrating", "abstract",
                        "author", "mtime", "ipath" };

static const int NTYPES = 9;
static const int MINITEMS = 8;  // for now, that ipath might be missing!

static const int RECOLLMAX = 2000;
  
class recollparser {
    
public:
    recollparser( QByteArray input, int nUseMax );//parse from input string
    recollparser( QStringList data );           //copy from pre-existing
    recollparser() { }                          // default null-constructor
    ~recollparser() { }

    void init( int nUseMax = 0 );               //perform inits

    QStringList data()          { return m_data; }
    QString mimeType()          { return m_data[ eMIMETYPE ]; }
    QString title()             { return m_data[ eTITLE ]; }
    const QUrl& URL()           { return m_URL; }  
    QString abstract()          { return m_data[ eABSTRACT ]; }
    QString author()            { return m_data[ eAUTHOR ]; }
    const QFileInfo& file()     { return m_file; }
    QString recollFileName()    { return m_data[ eFILENAME ]; }
    QString recollURL()         { return m_data[ eURL ]; }
    QString fileName()          { return m_file.fileName(); }
    QString absolutePath()      { return m_file.absoluteFilePath(); }
    QString urlPath()           { return m_URL.toLocalFile(); }
    QString recollRelevStr()    { return m_data[ eRELEVANCE ]; }
    QString ipath()             { return m_data[ eIPATH ]; }
    QDate date();
    QString dateLocaleString()  { return date().toString( Qt::LocaleDate ); }
    QString dateLocaleLongString() { return date().toString( Qt::SystemLocaleLongDate ); }

    bool isFile()               { return m_file.isFile(); }
    bool isDir()                { return m_file.isDir(); }
    bool isHidden()             { return m_file.isHidden(); }
    bool hasTitle()             { return m_data[ eFILENAME ] != m_data[ eTITLE ]; }

    bool hasIPath()             { return m_data[ eIPATH ].length() > 0; }
    bool ipathIsNumber();
    bool ipathIsNumberColonNumber();
    int ipathToNumber();

    
    //TEST THIS FIRST!!
    int nItems()                { return m_nParsed; }
    bool isValid()              { return m_nParsed >= MINITEMS; }
         
    static QString fieldList();
    double recollRelevfloat();
    int recollRelevInt();
    
    // some more convenience functions
    double mapRelevToRange( double min, double max );
    static double mapFloatToRange( double rank, double min, double max );
    double mapSeqToRange( int seq, double min, double max );

protected:
    void copyData( QStringList& );
    
private:
    QFileInfo m_file;
    QUrl m_URL;
    
    QList< QByteArray > m_raw;
    QStringList m_data;

    int m_nMax;
    int m_nParsed;
};

#endif	/* RECOLLPARSER_H */

