/**** Recollrunner source


 */
#define KRUNNERBUG

#include <KRun>
#include <KIcon>
#include <KMenu>
//#include <KCModuleProxy>
#include <kwindowsystem.h>

#include <plasma/runnersyntax.h>
#include <plasma/theme.h>

#include <QAction>
#include <QMenu>
#include <QDebug>
#include <QMessageBox>

#include "recollrunner.h"
#include "recollworker.h"
#include "config.h"
#include "cmdstrings.h"
#include "recollparser.h"
#include "recollViewer.h"
#include "recollrunnerconfig.h"


class RunnerSyntax;

recollrunner::recollrunner(QObject *parent, const QVariantList& args) :
                                    Plasma::AbstractRunner(parent, args) {    
    Q_UNUSED(args)
            
    setObjectName("recollrunner");
    setHasRunOptions( true );

    m_viewer = new recollViewer();
}

recollrunner::~recollrunner() {
}

void recollrunner::match(Plasma::RunnerContext &context) {
    m_matchContext = context;

    QString term = context.query();
    if (term.length() < 3) {
        return;
    }

    if  ( m_properties.m_bWantPrefix ) {
        qDebug() << term << "//" << m_properties.m_thePrefix;
        if  ( term.startsWith( m_properties.m_thePrefix  ))
            term.remove( 0, m_properties.m_thePrefix.length() );
        else
            return;
    }
    
    emit startMatching();

    m_viewer->registerUserQuery( term );

    RecollAsyncWorker *recoll = new RecollAsyncWorker( this, context, icon() );
    recoll->setOptions( &m_properties );
    
    //TODO
    // do this async even though it is very quick
    QString cmd = RECOLLCMD;

    QStringList args;
    QStringList partial;
    
    args << RECOLLCLI;
    
    if (m_properties.m_bFilenamesOnly) {
        args << RECOLLFILENAMES;
    } else {
        if (m_properties.m_queryType == OR)
            args << RECOLLOR;
        else if (m_properties.m_queryType == AND)
            args << RECOLLAND;
        else {
        };
    }
    
    partial = args;

    // This did not work in CLI to GUI!!
    switch (m_properties.m_sortType) {
        case NAME:
            args << RECOLLSORTFIELD;
            args << CMDS[ eTITLE ];
            break;
        case MIMETYPE:
            args << RECOLLSORTFIELD;
            args << CMDS[ eMIMETYPE ];
            break;
        case DATE:
            args << RECOLLSORTFIELD;
            args << CMDS[ eDATE ];
            args << RECOLLSORTREV;      // most recent first
            break;
        case RANK:
        default:
            break;
    }

    args << RECOLLFIELDS;
    args << recollparser::fieldList();
    args << RECOLLQUERY;
    args << term;       // quotes not needed in start(arg, argv) model

    /* Does not work!
    if  ( m_properties.m_bRestrictHits && m_properties.m_nMaxHits > 0 ) {
        partial << RECOLLMAXHITS;
        partial << QString::number( m_properties.m_nMaxHits );
    }*/
    partial << RECOLLQUERY;
    if (m_properties.m_bFilenamesOnly) {
        partial << term;
    }
    else {
        QString quoted;
        quoted = "'";
        quoted += term;
        quoted += "'";
        partial << quoted;
    }

    qDebug() << cmd << args;
    recoll->startWorker( cmd, args, partial );
    recoll->waitForFinished();
    
    delete recoll;
}

void recollrunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match) {
    Q_UNUSED(context)
    
    QStringList runit = match.data().toStringList();
 
    //QAction* act = match.selectedAction();
    if  ( m_properties.m_runType == VIEWER) { //( act == m_aview ) {
        emit runviewer( context, match );
        return;
    }

    runIt( runit[0] );
}

void recollrunner::runIt( QString toRun ) {
    qDebug() << toRun;
    
    if  ( toRun.indexOf( EXECCMD ) == 0) {
        // should have done this for me!
        KRun::runCommand( toRun.mid( EXECCMD.length() ), 0 );
    }
    else {
        (void) new KRun( toRun, 0 );
    }
}

void recollrunner::init() {
    //setHasConfigurationInterface( true );
    reloadConfiguration();
    
    connect( this, SIGNAL( teardown() ), this, SLOT( cleanUp() ));
    connect( this, SIGNAL( runviewer( Plasma::RunnerContext, Plasma::QueryMatch)),
            this, SLOT( viewer(Plasma::RunnerContext, Plasma::QueryMatch)));
    connect( this, SIGNAL( rerunviewer( Plasma::RunnerContext )),
            this, SLOT( reviewer(Plasma::RunnerContext )));
    connect( m_viewer, SIGNAL( research( QString) ),
            this, SLOT( reconfigureAndResearch( QString) ));

    //m_aview = addAction( "ACTION_PREVIEW", KIcon( "document-preview" ), i18n( "View file properties") );
    //m_aconfig = addAction( "ACTION_CONFIG", KIcon( "configure"), i18n("Reconfigure and re-search") );
    //m_aconfig->setToolTip( "Reconfigure and re-search");

#ifdef KRUNNERBUG
    // THIS IS TEMPORARY UNTIL PROBLEM FIXED
    //m_adummy = addAction( "DUMMY", QIcon(), i18n( "" ));
    //m_adummy->setIconVisibleInMenu( false );
    //m_adummy->setVisible(false);
    //m_adummy->setEnabled(false);
#endif // KRUNNERBUG

    m_viewer->setWindowIcon( icon() );
}

void recollrunner::cleanUp() {
    iconCache.clear();
}

void recollrunner::reloadConfiguration() {
    m_properties = loadConfig();
    
    QList<Plasma::RunnerSyntax> syntaxes;
    setSyntaxes( syntaxes );       // Empty, clears them
    if  ( m_properties.m_bWantPrefix ) {
        QString syntax = m_properties.m_thePrefix;
        syntax += " :q:";
        syntaxes.append( Plasma::RunnerSyntax( syntax, syntax + i18n( "to search recoll index for :q:" )));
        setSyntaxes( syntaxes );
    }
}

/*QList<QAction*> recollrunner::actionsForMatch(const Plasma::QueryMatch &match)
{
    Q_UNUSED( match )

    QList<QAction*> ret;
    //ret << m_aopen;
    ret << m_aview;
    //ret << m_aconfig;
    ret << m_adummy;

    return ret;
}*/

configAndQueryDlg::configAndQueryDlg( QWidget* parent, QString oldQuery, KIcon icon )
                                                :KCMultiDialog( parent ) {
    m_editQuery = new QLineEdit();
    m_editQuery->setAlignment( Qt::AlignTop );
    m_editQuery->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Preferred );
    m_editQuery->setFrame( true );
    m_query = oldQuery;
    m_editQuery->setText( oldQuery );
    
    connect( this, SIGNAL( configCommitted() ), this, SLOT( requestReview() ) );
    connect( this, SIGNAL( configCommitted() ), this, SLOT( close()));
    connect( this, SIGNAL( configCommitted() ), this, SLOT( deleteLater() ));
    connect( this, SIGNAL( cancelClicked() ), this, SLOT( deleteLater() ));
    
    QVBoxLayout* vb = new QVBoxLayout();
    vb->addWidget( m_editQuery );
    QLabel* lb = new QLabel();
    lb->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    lb->setAlignment( Qt::AlignTop );
    lb->setText( queryHelp() );
    vb->addWidget( lb ); 
    QWidget* wg = new QWidget();
    wg->setLayout( vb );
    KPageWidgetItem* ed = addPage( wg, "New Query" );
    ed->setIcon( icon );    
    
    KPageWidgetItem* pg = addModule( "recollrunnerconfig" );
    pg->setIcon( KIcon( "configure"));
    
    //m_cfg = (recollrunnerconfig*)  ((KCModuleProxy*) pg->widget())->realModule();
    //connect( m_editQuery, SIGNAL( textEdited( QString )), m_cfg, SLOT( forceChanged() ));
    
    m_icon = icon;
    
    setCurrentPage( ed );
    
    show();
    resize( pg->widget()->minimumSize() );
}

void configAndQueryDlg::requestReview() {
    m_query = m_editQuery->text();
    emit newView( m_query );
}

void configAndQueryDlg::busyIt() {
    qDebug() << "busy it";
    currentPage()->setEnabled( false );
    setCursor( QCursor( Qt::WaitCursor ));
}

QString configAndQueryDlg::queryHelp() {
    return i18n("<b>Recoll Query Syntax</b><hr>") + 
            i18n("<b>keyword-1 keyword-2</b> implies <i>AND</i>.<br>") +
            i18n("<b>keyword-1 OR keyword-2</b> has precedence.<br>") +
            i18n("<b>~keyword</b> means <i>NOT</i>.<br>") +
            i18n("<b>\"Quoted phrase\"</b> searches for the phrase.<p>") +
            i18n("<b>field-name:keyword</b> searches recoll's data fields:<p>") +
            i18n("Examples are:<ul>") +
            i18n("<li><b>title</b> document title or filename if none</li>") +
            i18n("<li><b>author</b> if document has one such as email <i>from</i></li>") +
            i18n("<li><b>filename</b> or <b>url</b>&mdash;document path</li>") +
            i18n("<li><b>ipath</b> internal archive path if any</li>") +
            i18n("<li><b>mtype</b> document mime type</li></ul><hr>") +
            i18n("<b>Query file names:</b><br>") +
            i18n("Capitalized implies <b>Filename</b>.<br>") +
            i18n("Not capitalized implies <b>*filename*</b>, case insensitive.<p>");
} 

void recollrunner::reconfigureAndResearch( QString query ) {    
    qDebug() << "Loading configuration/query dialog!";
    KCMultiDialog* dlg = new configAndQueryDlg( m_viewer, query, KIcon( icon() ) );
    connect( dlg, SIGNAL( newView( QString ) ), this, SLOT( researchAndView( QString ) ) );
    connect( dlg, SIGNAL( cancelClicked() ), this, SLOT( reshowViewer() ) );
    connect( this, SIGNAL( startMatching() ), dlg, SLOT( busyIt()));
}

/*void recollrunner::configBusy() {
    qDebug() << "setting config busy";
    m_Dlg->setEnabled( false );
    m_Dlg->setCursor( QCursor( Qt::WaitCursor ));
}**/

void recollrunner::research() {
    reloadConfiguration();
    QString term = m_matchContext.query();
    qDebug() << "Redo Match for " << term << "!!";

    /*Plasma::RunnerManager* manager = new Plasma::RunnerManager( this );
    manager->setSingleModeRunnerId( id() );
    manager->setupMatchSession();
    qDebug() << manager->execQuery( term, name() );
    manager->matchSessionComplete();*/
    //m_matchContext.reset();
    //m_matchContext.setQuery( term );
    //match( m_matchContext );
}

void recollrunner::researchAndView( QString term ) {
    reloadConfiguration();
    
    qDebug() << "Redo Match for " << term << "!!";

    /*Plasma::RunnerManager* manager = new Plasma::RunnerManager( this );
    manager->setSingleModeRunnerId( id() );
    manager->setupMatchSession();
    qDebug() << manager->execQuery( term, name() );
    manager->matchSessionComplete();*/
    m_matchContext.reset();
    m_matchContext.setQuery( term );
    performMatch( m_matchContext );
    
    emit rerunviewer( m_matchContext );
}


void recollrunner::createRunOptions(QWidget *widget) {    
    KMenu* menu = new KMenu( widget );
    
    // theme it, borrowed from runner applet code
    QPalette palette = menu->palette();
    palette.setColor(QPalette::WindowText, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
    palette.setColor(QPalette::ButtonText, Plasma::Theme::defaultTheme()->color(Plasma::Theme::ButtonTextColor));
    palette.setColor(QPalette::Window, Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
    palette.setColor(QPalette::Button, Plasma::Theme::defaultTheme()->color(Plasma::Theme::ButtonBackgroundColor));
    menu->setPalette( palette );
  
    QActionGroup* ag = new QActionGroup( this );
    ag->setExclusive( true );
    
    // this will appear grayed on many themes, however
    menu->addTitle( i18n( "Run action" ));
    
    QAction* ao = menu->addAction( KIcon( "document-open"), i18n( "Open file") );
    ag->addAction( ao );
    ao->setCheckable( true );
    ao->setChecked( m_properties.m_runType == OPENFILE );
    connect( ao, SIGNAL( toggled( bool )), this, SLOT( setRunOpen( bool)));
    
    QAction* av = menu->addAction( KIcon( "document-preview"), i18n( "Viewer") );
    ag->addAction( av );
    av->setCheckable( true );
    av->setChecked( m_properties.m_runType == VIEWER );
    connect( av, SIGNAL( toggled( bool )), this, SLOT( setRunViewer( bool)));
    
    menu->addSeparator();
    
    QAction* ac = menu->addAction( KIcon( "configure"), i18n( "Configure"));
    connect( ac, SIGNAL( triggered()), this, SLOT( configDlg()));
    
    menu->exec( QCursor::pos());
}

void recollrunner::configDlg(QWidget* widget) {
    qDebug() << "Loading configuration dialog!";
    KCMultiDialog* dlg = new KCMultiDialog( widget);
    connect( dlg, SIGNAL( configCommitted() ), this, SLOT( research() ) );
    connect( dlg, SIGNAL( configCommitted() ), dlg, SLOT( deleteLater() ));
    connect( dlg, SIGNAL( cancelClicked() ), dlg, SLOT( deleteLater() ));
    KPageWidgetItem* pg = dlg->addModule( "recollrunnerconfig");
    dlg->show();
    dlg->resize( pg->widget()->minimumSize() );
}

void recollrunner::viewer( const Plasma::RunnerContext& context, const Plasma::QueryMatch &match) {
    suspendMatching( true );
    m_viewer->runView( context, match, &m_properties );
    suspendMatching( false );
}

void recollrunner::reviewer( const Plasma::RunnerContext& context ) {
    Plasma::QueryMatch nullquery(this);
    
    suspendMatching( true );
    m_viewer->runView( context, nullquery, &m_properties );
    suspendMatching( false );
}

void recollrunner::reshowViewer() {
    m_viewer->show();
}



#include "recollrunner.moc"
