
#ifndef RECOLLRUNNER_H
#define RECOLLRUNNER_H

#include <plasma/abstractrunner.h>
#include <plasma/runnercontext.h>
#include <plasma/runnermanager.h>
#include <plasma/querymatch.h>
#include <plasma/package.h>

#include <QStandardItemModel>
#include <QLineEdit>

#include <KCMultiDialog>

#include "recollworker.h"
#include "config.h"

const QString ACTION_OPEN   = "OPENFILE";
const QString ACTION_CONFIG = "RECONFIG";

class RecollAsyncWorker;
class recollrunnerconfig;
class recollViewer;

class configAndQueryDlg : public KCMultiDialog {
    Q_OBJECT
    
public:
    configAndQueryDlg( QWidget* parent, QString oldQuery, KIcon icon );
    ~configAndQueryDlg() { }

    
protected:
    static QString queryHelp();

private slots:
    void requestReview();
    void busyIt();
    
protected:
    QString m_query;
    QLineEdit* m_editQuery;
    QWidget* m_widget;
    KIcon m_icon;
    
    recollrunnerconfig* m_cfg;
    
Q_SIGNALS:
    void newView(QString);
    
    
};

// Define our plasma Runner
class recollrunner : public Plasma::AbstractRunner {
    Q_OBJECT

public:
    // Basic Create/Destroy
    recollrunner( QObject *parent, const QVariantList& args );
    ~recollrunner();

    void match(Plasma::RunnerContext &context);
    void run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match);

    //QList<QAction*> actionsForMatch(const Plasma::QueryMatch &match);
    void createRunOptions(QWidget *widget);
    
public slots:
    void reloadConfiguration();
    void reconfigureAndResearch( QString );
    void research();
    void researchAndView( QString );
    
protected slots:
    void init();
    void cleanUp();
    void runIt( QString toRun );
    void viewer( const Plasma::RunnerContext &context, const Plasma::QueryMatch &match);
    void reviewer( const Plasma::RunnerContext &context );
    void reshowViewer();
    void configDlg( QWidget* = 0 );
    void setRunOpen( bool b ) { if ( b ) m_properties.m_runType = OPENFILE; }
    void setRunViewer( bool b ) { if ( b ) m_properties.m_runType = VIEWER; }
    //void configBusy();

Q_SIGNALS:
    void runviewer( const Plasma::RunnerContext&, const Plasma::QueryMatch& );
    void rerunviewer( const Plasma::RunnerContext& );
    void startMatching();

protected:
    RRSettings m_properties;
    Plasma::RunnerContext m_matchContext;

    QAction* m_aopen;
    QAction* m_aview;
    QAction* m_aconfig;
    QAction* m_adummy;
    
    recollViewer* m_viewer;
};


// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_RUNNER(recollrunner, recollrunner)


#endif
