/* 
 * File:   recollrunnerconfig.cpp
 * Author: david
 * 
 * Created on August 4, 2010, 5:03 PM
 */

#include "recollrunnerconfig.h"

recollrunnerconfigDlg::recollrunnerconfigDlg( QWidget *parent ):QWidget( parent ) {
    UI.setupUi( this );
    show();
}

recollrunnerconfigDlg::~recollrunnerconfigDlg() {
}

void recollrunnerconfigDlg::setup( RRSettings& settings ) {
    switch( settings.m_queryType ) {
        case AND:      UI.radioButtonAnd->setChecked( true );       break;
        case OR:       UI.radioButtonOr->setChecked( true );        break;
        case QUERY:
        default:       UI.radioButtonQueryLang->setChecked( true ); break;
    }
    switch( settings.m_sortType ) {
        case NAME:     UI.radioButtonSortName->setChecked( true );  break;
        case MIMETYPE: UI.radioButtonSortMime->setChecked( true );  break;
        case DATE:     UI.radioButtonSortDate->setChecked( true );  break;
        case RANK:
        default:       UI.radioButtonSortRank->setChecked( true );  break;
    }
    switch( settings.m_runType ) {
        case VIEWER:   UI.radioButtonRunViewer->setChecked( true ); break;
        case OPENFILE:
        default:       UI.radioButtonRunOpen->setChecked( true );   break;
    }
    UI.checkBoxMax->setChecked( settings.m_bRestrictHits );
    UI.spinBoxMax->setValue( settings.m_nMaxHits );
    UI.spinBoxMax->setEnabled( settings.m_bRestrictHits );
    UI.checkBoxMinRank->setChecked( settings.m_bRestrictRank );
    UI.spinBoxMinRank->setValue( settings.m_nMinRank );
    UI.spinBoxMinRank->setEnabled( settings.m_bRestrictRank);
    UI.checkBoxFilenames->setChecked( settings.m_bFilenamesOnly );
    UI.checkBoxHidden->setChecked( settings.m_bHiddenFiles );
    UI.checkBoxTypes->setChecked( settings.m_bRestrictTypes );
    UI.lineEditTypes->setText( settings.m_fileTypes );
    UI.lineEditTypes->setEnabled( settings.m_bRestrictTypes );
    UI.checkBoxPatterns->setChecked( settings.m_bRestrictPatterns );
    UI.radioButtonPatternAnd->setChecked( settings.m_bAllPatterns );
    UI.radioButtonPatternOr->setChecked( !settings.m_bAllPatterns );
    UI.lineEditPatterns->setText( settings.m_filePatterns );
    UI.lineEditPatterns->setEnabled( settings.m_bRestrictPatterns );
    UI.radioButtonPatternOr->setEnabled( settings.m_bRestrictPatterns );
    UI.radioButtonPatternAnd->setEnabled( settings.m_bRestrictPatterns );
    UI.checkBoxNonfiles->setChecked( settings.m_bNonFiles );
    UI.checkBoxTag->setChecked( settings.m_bTag );
    UI.checkBoxRecoll->setChecked( settings.m_bRunRecoll );
    UI.checkBoxKIO->setChecked( settings.m_bRunKIO );
    UI.checkBoxNoArchives->setChecked( settings.m_bNoArchives );
    UI.checkBoxPrefix->setChecked( settings.m_bWantPrefix );
    UI.lineEditPrefix->setText( settings.m_thePrefix );
    UI.lineEditPrefix->setEnabled( settings.m_bWantPrefix );
}

RRSettings recollrunnerconfigDlg::getSettings() {
    RRSettings settings;

    if  ( UI.radioButtonAnd->isChecked() )
        settings.m_queryType = AND;
    else if ( UI.radioButtonOr->isChecked() )
         settings.m_queryType = OR;
    else settings.m_queryType =  QUERY;

    if  ( UI.radioButtonSortMime->isChecked() )
        settings.m_sortType = MIMETYPE;
    else if  ( UI.radioButtonSortName->isChecked() )
        settings.m_sortType = NAME;
    else if  ( UI.radioButtonSortDate->isChecked() )
        settings.m_sortType = DATE;
    else settings.m_sortType =  RANK;

    if  ( UI.radioButtonRunViewer->isChecked() )
        settings.m_runType = VIEWER;
    else settings.m_runType =  OPENFILE;
    
    settings.m_bFilenamesOnly = UI.checkBoxFilenames->isChecked();
    settings.m_bRestrictHits = UI.checkBoxMax->isChecked();
    settings.m_nMaxHits = UI.spinBoxMax->value();
    settings.m_bRestrictRank = UI.checkBoxMinRank->isChecked();
    settings.m_nMinRank = UI.spinBoxMinRank->value();
    settings.m_bHiddenFiles = UI.checkBoxHidden->isChecked();
    settings.m_bRestrictTypes = UI.checkBoxTypes->isChecked();
    settings.m_fileTypes = UI.lineEditTypes->text();
    settings.m_bRestrictPatterns = UI.checkBoxPatterns->isChecked();
    settings.m_bAllPatterns = UI.radioButtonPatternAnd->isChecked();
    settings.m_filePatterns = UI.lineEditPatterns->text();
    settings.m_bNonFiles = UI.checkBoxNonfiles->isChecked();
    settings.m_bTag = UI.checkBoxTag->isChecked();
    settings.m_bRunRecoll = UI.checkBoxRecoll->isChecked();
    settings.m_bRunKIO = UI.checkBoxKIO->isChecked();
    settings.m_bNoArchives = UI.checkBoxNoArchives->isChecked();
    settings.m_bWantPrefix = UI.checkBoxPrefix->isChecked();
    settings.m_thePrefix = UI.lineEditPrefix->text();

    return settings;
}
// integerate UI
K_PLUGIN_FACTORY(MyConfigFactory,
                   registerPlugin<recollrunnerconfig>();
                 )
K_EXPORT_PLUGIN(MyConfigFactory("recollrunnerconfig", "recollrunnerconfig"))

recollrunnerconfig::recollrunnerconfig(QWidget* parent, const QVariantList& args):
        KCModule(MyConfigFactory::componentData(), parent, args)
{
    ui = new recollrunnerconfigDlg( this );

    // Seems I allways need some trick like this
    QSize sz = ui->size();
    setMinimumSize( sz );
    
    load();
    connect(ui->UI.radioButtonQueryLang, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.radioButtonAnd, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.radioButtonOr, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxMax, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.spinBoxMax, SIGNAL(valueChanged(int)), SLOT(changed()));
    connect(ui->UI.checkBoxFilenames, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxHidden, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxTypes, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxPatterns, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.lineEditTypes, SIGNAL( textChanged(QString)), SLOT(changed()));
    connect(ui->UI.lineEditPatterns, SIGNAL( textChanged(QString)), SLOT(changed()));
    connect(ui->UI.radioButtonPatternAnd, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.radioButtonPatternOr, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxNonfiles, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxTag, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxRecoll, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxKIO, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxMinRank, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.spinBoxMinRank, SIGNAL(valueChanged(int)), SLOT(changed()));
    connect(ui->UI.radioButtonSortRank, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.radioButtonSortName, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.radioButtonSortMime, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.radioButtonSortDate, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxNoArchives, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.checkBoxPrefix, SIGNAL(toggled(bool)), SLOT(changed()));;
    connect(ui->UI.lineEditPrefix, SIGNAL( textChanged(QString)), SLOT(changed()));
    connect(ui->UI.radioButtonRunOpen, SIGNAL(toggled(bool)), SLOT(changed()));
    connect(ui->UI.radioButtonRunViewer, SIGNAL(toggled(bool)), SLOT(changed()));
}

recollrunnerconfig::~recollrunnerconfig() {
    delete ui;
}

void recollrunnerconfig::save() {
    saveConfig( ui->getSettings() );

    emit changed( false );
}

void recollrunnerconfig::load() {
    KCModule::load();

    RRSettings settings = loadConfig();
    ui->setup( settings );

    refresh();
    emit changed( false );
}

void recollrunnerconfig::defaults() {
    RRSettings settings;

    settings.m_queryType = (eQUERY) DEFAULTQUERY;
    settings.m_sortType = (eSORT) DEFAULTSORT;
    settings.m_runType = (eRUN) DEFAULTRUNTYPE;
    settings.m_bFilenamesOnly = DEFAULTFILENAMES;
    settings.m_bRestrictHits = DEFAULTRESTRICTFILES;
    settings.m_nMaxHits = DEFAULTMAXFILES;
    settings.m_bRestrictRank = DEFAULTRESTRICTRANK;
    settings.m_nMinRank = DEFAULTMINRANK;
    settings.m_bRestrictTypes = DEFAULTRESTRICTTYPES;
    settings.m_fileTypes = DEFAULTTYPES;
    settings.m_bRestrictPatterns = DEFAULTRESTRICTPATTERNS;
    settings.m_bAllPatterns = DEFAULTALLPATTERNS;
    settings.m_filePatterns = DEFAULTPATTERNS;
    settings.m_bNonFiles = DEFAULTNONFILES;
    settings.m_bTag = DEFAULTTAG;
    settings.m_bRunRecoll = DEFAULTRUNRECOLL;
    settings.m_bRunKIO = DEFAULTRUNKIO;
    settings.m_bNoArchives = DEFAULTNOARCHIVES;
    settings.m_bWantPrefix = DEFAULTWANTPREFIX;
    settings.m_thePrefix = DEFAULTTHEPREFIX;

    ui->setup( settings );

    emit changed(true);
}

void recollrunnerconfig::refresh() {

}
#include "recollrunnerconfig.moc"